@@ -1,6 +1,8 @@
 import cPickle, os
 import gtk, gobject
-from deskbar import MAX_HISTORY, HISTORY_FILE, MAX_RESULTS_PER_HANDLER
+from deskbar import MAX_HISTORY, HISTORY_FILE 
+from deskbar.Utils import load_icon
+from gettext import gettext as _
 
 class DeskbarHistoryIter : 
 	"""An iter type to iterate over a DeskbarHistory.
@@ -22,29 +24,30 @@ class DeskbarHistoryIter :
 			raise StopIteration
 		return item
 
-# The sort function ids
-SORT_CHRONOLOGICAL = 1
+empty_history_icon = load_icon(gtk.STOCK_STOP)
 
-class SortedDeskbarHistory (gtk.TreeModelSort):	
-	def __init__(self):
-		gtk.TreeModelSort.__init__(self, get_deskbar_history())
-		self.set_sort_func(SORT_CHRONOLOGICAL, self.on_sort_chronological)
-		self.set_sort_column_id(SORT_CHRONOLOGICAL, gtk.SORT_ASCENDING)
+class EmptyHistoryMatch:
+	def get_icon(self):
+		return empty_history_icon
 	
-	def set_sort_order(self, order):
-		#self.set_sort_column_id(SORT_CHRONOLOGICAL, order)
-		# Somehow this triggers a segfault !
+	def action(self, text=None):
 		pass
-	
-	def on_sort_chronological(self, model, iter1, iter2):
-		return 1
-
+		
+	def get_name(self, text=None):
+		return {"msg": _("No History")}
+		
+	def get_verb(self):
+		return "%(msg)s"
+		
 class DeskbarHistory (gtk.ListStore) :
 	"""
-	Iteraating over a DeskbarHistory with a for loop returns (text,match) pairs.
+	Iterating over a DeskbarHistory with a for loop returns (text,match) pairs.
 	Keeps an internal pointer to a history index which you can move with up(), down(),
 	and reset(). You retrieve the item in question by get_history().
 	
+	Text-Match pairs are stored in column 0, while a timestamp (a simple counter really)
+	is stored in column 1. The timestamp is used for sorting purposes.
+	
 	Signals:
 		"changed" : emitted when the internal pointer has changed
 	"""
@@ -54,14 +57,37 @@ class DeskbarHistory (gtk.ListStore) :
 	}
 	
 	def __init__ (self):
-		gtk.ListStore.__init__ (self, gobject.TYPE_PYOBJECT)
+		gtk.ListStore.__init__ (self, gobject.TYPE_PYOBJECT, gobject.TYPE_INT)
+		self.set_sort_column_id (1, gtk.SORT_ASCENDING)
+		self.set_sort_func (1, self.sort_matches)
+		self.timestamp = 0
 		self._index = -1
 		
 	def __iter__ (self):
 		return DeskbarHistoryIter (self)				
+	
+	def set_sort_order (self, order):
+		"""order should be one of gtk.SORT_{ASCENDING,DESCENDING}"""
+		self.set_sort_column_id (1, order)
+	
+	def sort_matches (self, model, iter1, iter2):
+		if self[iter1][1] > self[iter2][1] :
+			return 1
+		else:
+			return -1
+	
+	def clear (self):
+		gtk.ListStore.clear(self)
+		self.timestamp = 0
+		self.append(("", EmptyHistoryMatch()))
+		self._index = -1
+		self.emit("changed")
+	
+	def clear_stub(self):
+		if len(self) == 1 and self[self.get_iter_first()][0][1].__class__ == EmptyHistoryMatch:
+			gtk.ListStore.clear(self)
 		
 	def load (self, module_list):
-		print 'Loading History'
 		new_history = []
 		try:
 			saved_history = cPickle.load(file(HISTORY_FILE))
@@ -84,18 +110,35 @@ class DeskbarHistory (gtk.ListStore) :
 						
 					match = modctx.module.deserialize(match_class, serialized)
 					if match != None:
-						new_history.append ([(text, match)])
+						new_history.append ((text, match))
+		except IOError:
+			# There's probably no history file
+			pass
+			
 		except Exception, msg:
 			return
 		
+		self.clear()
 		if len(new_history) > 0:
-			self.clear()
+			self.clear_stub()
 			for hist in new_history:
 				self.append (hist)
-				
+
 	def save (self):
+		# FIXME: We do not save the timestamp. 
+		# This is because we can't break history 
+		# file format in the 2.14 series.
+		# Is it needed anyway?
+		
+		# We need ascending sort order for save/load to work correctly
+		column_id, old_order = self.get_sort_column_id () # Store the old sort order
+		self.set_sort_column_id (1, gtk.SORT_ASCENDING)
+		
 		save = []
 		for text, match in self:
+			if match.__class__ == EmptyHistoryMatch:
+				return
+				
 			hsh = match.get_hash(text)
 			save.append((text, str(match.get_handler().__class__), str(match.__class__), match.serialize()))
 			
@@ -104,12 +147,28 @@ class DeskbarHistory (gtk.ListStore) :
 		except Exception, msg:
 			print 'Error:History.save:%s', msg
 		pass
+		
+		# Restore sort order
+		self.set_sort_column_id (1, old_order)
+	
+	def append (self, match_obj):
+		gtk.ListStore.append (self, (match_obj, self.timestamp))
+		self.timestamp = self.timestamp + 1
+	
+	def prepend (self, match_obj):
+		print "ERROR: DeskbarHistory does not support prepending of matches, use append() instead."
+		raise Exception
 	
 	def add (self, text, match):
+		if match.__class__ == EmptyHistoryMatch:
+			return
+			
+		self.clear_stub()
+		
 		copy_match = True
 		for idx, val in enumerate(self):
 			htext, hmatch = val
-			if (text, match.__class__) == (htext, hmatch.__class__):
+			if (match.get_hash(text), match.__class__) == (hmatch.get_hash(htext), hmatch.__class__):
 				match = self[self.get_iter_from_string (str(idx))][0][1]
 				self.remove (self.get_iter_from_string (str(idx)))
 				copy_match = False
@@ -120,12 +179,13 @@ class DeskbarHistory (gtk.ListStore) :
 			if copy != None:
 				match = copy
 				
-		self.prepend ([(text, match)])
+		self.append ((text, match))
 		if len(self) > MAX_HISTORY:
 			# Remove the last element
 			last = self.get_iter_from_string (str(len(self) - 1))
 			self.remove (last)
-		self._index = -1
+
+		self.reset()
 		self.save()
 	
 	def up(self):
