@@ -3,7 +3,7 @@ from gettext import gettext as _
 
 import urllib, cgi
 import gnomevfs
-import deskbar.Handler
+import deskbar.Handler, deskbar
 import xml.dom.minidom
 
 YAHOO_API_KEY = 'deskbar-applet'
@@ -40,9 +40,9 @@ class YahooHandler(deskbar.Handler.AsyncHandler):
 		deskbar.Handler.AsyncHandler.__init__(self, "yahoo.png")
 		self.server = None
 
-	def query(self, qstring, qmax):
+	def query(self, qstring):
 		# Just to ensure we don't bork anything
-		qmax = min (qmax, MAX_QUERIES)
+		qmax = min (deskbar.DEFAULT_RESULTS_PER_HANDLER, MAX_QUERIES)
 
 		# Delay before we query so we *don't* make four queries
 		# "s", "sp", "spa", "spam".
